/*
 * File:       <compressTex.C>
 * Author:     <Remigiusz Trzaska>
 * e-mail:     <<remek@>>
 * Created:    <Tue Mar 18 1997>
 * Time-stamp: <97/03/21 12:53:57 remek>
 * Copyright Remigiusz Trzaska 1997 NPAC, Syracuse University
 *
 */

#include "Memory.h"
#include "wv.h"
#include "stdio.h"

void
readPPM(FILE *ppm, unsigned char*& data, int& dimx, int& dimy)
{
   int i;

   //read "P6\n"
   for(i=0; i<3; i++)
      fgetc(ppm);
   
   unsigned char c;
   unsigned char junk;
   int foundComment;
   
   do
   {
      c = fgetc(ppm);
      foundComment = c=='#';
      if(foundComment)
	 do 
	    junk = fgetc(ppm);
	 while(junk!='\n');
      else
	 ungetc(c, ppm);
   }
   while(foundComment);

   fscanf(ppm, "%d", &dimx);
   fscanf(ppm, "%d", &dimy);

   //ignore "255\n"
   int _;
   fscanf(ppm, "%d", &_);
   //get rid of a newline
   fgetc(ppm);

   NewVector(data, dimx*dimy*3, unsigned char);
   fread(data, sizeof(unsigned char), (long)dimx*dimy*3, ppm);
}

void
savePPM(FILE *ppm, unsigned char* data, int dimx, int dimy)
{
   //write header
   fprintf(ppm, "P6\n# CREATOR: uncompressTex by Remek Trzaska 1997\n"
	   "%d %d\n255\n", dimx, dimy);
   fwrite(data, sizeof(unsigned char), (long)dimx*dimy*3, ppm);
}

void
compressData(unsigned char* data, int dimx, int dimy,
	     unsigned char*& outStream, int& usedLength, int& returnedQuality)
{
   //allocate space for compressed image
   //pesimistic assumption: the sizes are equal
   NewVector(outStream, dimx*dimy*3, unsigned char);
   usedLength = dimx*dimy*3;
   
   twv_t* compressor = new_twv(dimy, dimx);

   returnedQuality = enc_twv(compressor,
			     data,
			     outStream,
			     &usedLength,
			     returnedQuality);

   if(returnedQuality<0)
   {
      fprintf(stderr, "Compression failed! Returned stream lenght is %d\n",
	      usedLength);
      exit(1);
   }
   
   delete_twv(&compressor);
}

void
saveStream(FILE* outFile, unsigned char* stream,
	   int length, int dimx, int dimy)
{
   fwrite(&dimx, sizeof(int), 1, outFile);
   fwrite(&dimy, sizeof(int), 1, outFile);
   fwrite(stream, sizeof(unsigned char), length, outFile);
}


int
main(int argc, char* argv[])
{
   unsigned char* data;
   int dimx;
   int dimy;

   if(argc!=4)
   {
      fprintf(stderr, "Usage: %s infile outfile quality\n"
	      "\twhere\tinfile is a ppm image file\n"
	      "\t\toutfile is a name for the output file\n"
	      "\t\tquality is a number 1..11\n\n", argv[0]);
      return(-1);
   }
   
   FILE* inFile = fopen(argv[1], "rb");
   if(!inFile)
   {
      fprintf(stderr, "Cannot open the file: %s\nExiting...\n\n", argv[1]);
      return(-1);
   }

   FILE* outFile = fopen(argv[2], "wb");
   if(!outFile)
   {
      fprintf(stderr, "Cannot open the output file: %s\nExiting...\n\n",
	      argv[2]);
      return(-1);
   }

   readPPM(inFile, data, dimx, dimy);
   fclose(inFile);

   int quality = atoi(argv[3]);
   int usedLength;
   unsigned char* stream; //a place for the wavelet output

   compressData(data, dimx, dimy, stream, usedLength, quality);

   fprintf(stdout, "Compression status:\n\toutput stream length: %d\n"
	   "\treturned quality: %d\n\n", usedLength, quality);

   saveStream(outFile, stream, usedLength, dimx, dimy);
   
   fclose(outFile);
}
